@@ -1,4 +1,4 @@
-/*  Copyright (C) 2002 yvind Hagen */
+/*  Copyright (C) 2002-2004 yvind Hagen */
 
 /*  This file is part of gkleds. */
 /*  gkleds is free software; you can redistribute it and/or modify */
@@ -14,23 +14,29 @@
 /*  along with gkleds; if not, write to the Free Software */
 /*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA */
 
+#include "config.h"
 
 #if defined(WIN32)
-#if defined(GKLEDS_SELFCOMPILE)
-#include "gkrellm2/gkrellm.h"
-#include "gkrellm2/win32-plugin.h"
+#include <windows.h>
 #else
+#include <X11/Xlib.h>
+#include <X11/XKBlib.h>
+#include <X11/extensions/XTest.h>
+#include <X11/keysym.h>
+#endif
+
+#include <stdlib.h>
+#include <stdio.h>
+
+#if defined(WIN32)
 #include <src/gkrellm.h>
 #include <src/win32-plugin.h>
-#endif
 #else
 #include <gkrellm2/gkrellm.h>
 #endif
 
-#include "gkleds_conf.h"
-#include "gkleds_ind.h"
 #include "gkleds.h"
-#include "pixmaps/leds.xpm"
+#include "leds.xpm"
 
 static GkrellmMonitor gkleds_mon = {
   GKLEDS_NAME,		
@@ -91,6 +97,107 @@ static gint panel_expose_event(GtkWidget *widget, GdkEventExpose *ev){
   return FALSE;
 }
 
+static void gkleds_ind_conf(gkleds_indicator_conf *ind){
+  int i;
+#if defined(WIN32)
+  SHORT key_syms[GKLEDS_NUM_IND] = {VK_NUMLOCK, VK_CAPITAL, VK_SCROLL};
+#else
+  char *ind_name = NULL;
+  int j, mask;
+  int key_syms[GKLEDS_NUM_IND] = {XK_Num_Lock, XK_Caps_Lock, XK_Scroll_Lock};
+  char *lock_names[GKLEDS_NUM_IND] = {"Num Lock","Caps Lock", "Scroll Lock"};
+  XkbDescPtr xkb;
+#endif
+
+#ifdef GKLEDS_DEBUG
+  printf("configure_indicators: start\n"); fflush(stdout);
+#endif
+
+#if defined(WIN32)
+  for(i = 0; i < GKLEDS_NUM_IND; i++){
+    ind->idx[i] = i; 
+    ind->masks[i] = 1 << i;
+	ind->keys[i] = key_syms[i];
+  }
+#else  
+  /* we need a display handler to get/set indicator status */
+  ind->dpy = XOpenDisplay(NULL);
+  if(!ind->dpy){
+    printf("gkrellm::%s : unable to connect to the X server\n", 
+	   GKLEDS_NAME);
+    printf("gkrellm::%s : you won't be able to get/set indicator status\n", 
+	   GKLEDS_NAME);
+    return;
+  }
+  
+  /* get the keycode for set_indicators */
+  for(i = 0; i < GKLEDS_NUM_IND; i++)
+    ind->keys[i] = XKeysymToKeycode(ind->dpy, key_syms[i]);
+
+  /* get a keyboard structure */
+  xkb = XkbAllocKeyboard();
+  if(!xkb){
+    printf("gkrellm::%s : unable to allocate keyboard structure\n", 
+	   GKLEDS_NAME);
+    printf("gkrellm::%s : you won't be able to get/set indicator status\n", 
+	   GKLEDS_NAME);
+    XCloseDisplay(ind->dpy); ind->dpy = NULL;
+    return;
+  }
+
+  /* get the names(atoms) of the indicators */
+  if(XkbGetNames(ind->dpy, XkbIndicatorNamesMask, xkb) != Success){
+    printf("gkrellm::%s : unable to get indicator names from the X server\n", 
+	   GKLEDS_NAME);
+    printf("gkrellm::%s : you won't be able to get/set indicator status\n", 
+	   GKLEDS_NAME);
+    XkbFreeKeyboard(xkb, 0, True);
+    XCloseDisplay(ind->dpy); ind->dpy = NULL;
+    return;
+  }    
+
+  /* get the correct mask & shift for get_indicators */ 
+  for(i = 0; i < XkbNumIndicators; i++){
+    if(xkb->names->indicators[i])
+      ind_name = XGetAtomName(ind->dpy, xkb->names->indicators[i]);
+#ifdef GKLEDS_DEBUG
+  if(ind_name) printf("configure_indicators: %s\n", ind_name); fflush(stdout);
+#endif
+    for(j = 0; j < GKLEDS_NUM_IND; j++){
+      if(ind_name && !strcmp(lock_names[j], ind_name)){
+	if(XkbGetNamedIndicator(ind->dpy, xkb->names->indicators[i], &mask, 
+				NULL, NULL, NULL) == True){
+#ifdef GKLEDS_DEBUG
+  printf("configure_indicators: mask %d: %d\n", i, mask); fflush(stdout);
+#endif
+	  ind->masks[j] = 1 << mask;
+	  ind->idx[j] = mask;
+	}else{
+	  printf("gkrellm::%s : unable to get the indicator mask from the X "
+		 "server\n", GKLEDS_NAME);
+	  printf("gkrellm::%s : you won't be able to get/set indicator "
+		 "status\n", GKLEDS_NAME);
+	  XkbFreeKeyboard(xkb, 0, True);
+	  XCloseDisplay(ind->dpy); ind->dpy = NULL;
+	  return;
+	}
+      }
+    }
+    if(ind_name){ 
+      free(ind_name);
+      ind_name = NULL;
+    }
+  }
+
+  XkbFreeKeyboard(xkb, 0, True);
+#endif
+
+#ifdef GKLEDS_DEBUG
+  printf("configure_indicators: end\n"); fflush(stdout);
+#endif
+  return;
+}
+
 
 static void gkleds_ind_get(void){
   unsigned int states = 0;
@@ -475,9 +582,9 @@ static void gkleds_conf_create(GtkWidget *conf_vbox){
    "\tset any of the subimages to none if you want.\n"}; 
   char *about_text_locale = g_strdup_printf
     ("gkleds %s\n\n"
-     "Copyright (C) 2002 yvind Hagen\n"
+     "Copyright (C) 2002-2004 yvind Hagen\n"
      "oyvinha@ifi.uio.no\n"
-     "http://www.stud.ifi.uio.no/~oyvinha/gkleds\n\n"
+     "http://heim.ifi.uio.no/~oyvinha/gkleds\n\n"
      "Released under the GNU General Public Licence",
      GKLEDS_VERSION);
   char *about_text = g_locale_to_utf8(about_text_locale, -1 , 
@@ -934,6 +1041,15 @@ static void pm_idx_altered(GtkWidget *widget, gpointer data){
 }
 
 
+/* just hide how config copying is done */
+static void gkleds_conf_copy(gkleds_user_conf *from, gkleds_user_conf *to){
+
+  memcpy(to, from, sizeof(gkleds_user_conf));
+ 
+  return;
+}
+
+
 static void gkleds_conf_apply(void){
 
 #ifdef GKLEDS_DEBUG
